<?php

require_once 'Horde/IMAP/Search.php';

/**
 * The IMP_VFolder:: class provides the code necessary to handle virtual
 * folders in IMP.
 *
 * $Horde: imp/lib/VFolder.php,v 1.6.6.12 2006/01/01 21:28:55 jan Exp $
 *
 * Copyright 2004-2006 Michael Slusarz <slusarz@curecanti.org>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Michael Slusarz <slusarz@curecanti.org>
 * @since   IMP 4.0
 * @package IMP
 */
class IMP_VFolder {

    /**
     * Obtains the list of virtual folders for the current user.
     *
     * @return mixed
     */
    function getVFolderList()
    {
        $vfolder = $GLOBALS['prefs']->getValue('vfolder');
        return (empty($vfolder)) ? $vfolder : unserialize($vfolder);
    }

    /**
     * Saves the list of virtual folders for the current user.
     *
     * @access private
     */
    function _saveVFolderList()
    {
        $vfolder = empty($_SESSION['imp']['vfolder']) ? '' : serialize($_SESSION['imp']['vfolder']);
        $GLOBALS['prefs']->setValue('vfolder', $vfolder);
    }

    /**
     * Add a virtual folder for the current user.
     *
     * @param object &$ob     A IMAP_Search_Query object.
     * @param array &$search  The array of information used to build the
     *                        search UI page.
     * @param string $id      The vfolder ID to use.  If empty, an id is
     *                        automatically generated.
     *
     * @return string  The virtual folder ID.
     */
    function addVFolder(&$ob, $search, $id = null)
    {
        if (is_null($id)) {
            $id = base_convert(microtime() . mt_rand(), 10, 36);
        }
        $_SESSION['imp']['vfolder'][$id] = array(
            'ob' => serialize($ob),
            'search' => $search
        );
        IMP_VFolder::_saveVFolderList();

        return $id;
    }


    /**
     * Add a virtual trash folder for the current user.
     */
    function createVTrashFolder()
    {
        /* Delete the current Virtual Trash folder, if it exists. */
        $vtrash_id = $GLOBALS['prefs']->getValue('vtrash_id');
        if (!empty($vtrash_id)) {
            IMP_VFolder::deleteVFolder($vtrash_id);
        }

        if (!$GLOBALS['prefs']->getValue('use_vtrash')) {
            return;
        }

        /* Create Virtual Trash with new folder list. */
        require_once IMP_BASE . '/lib/Folder.php';
        $imp_folder = &IMP_Folder::singleton();
        $fl = $imp_folder->flist_IMP();
        $flist = array();
        foreach ($fl as $mbox) {
            if (!empty($mbox['val'])) {
                $flist[] = $mbox['val'];
            }
        }
        array_unshift($flist, 'INBOX');

        require_once 'Horde/IMAP/Search.php';

        $query = &new IMAP_Search_query();
        $query->deleted(true);

        $search = array('match'         => 'and',
                        'flag'          => array('1' => 2, '2' => 2, '3' => 2, '4' => '0'),
                        'folders'       => $flist,
                        'text'          => '',
                        'text_end'      => '',
                        'mbox'          => '',
                        'date'          => '',
                        'field'         => "",
                        'field_end'     => "",
                        'vfolder_label' => _("Virtual Trash"));

        if (empty($vtrash_id)) { 
            $vtrash_id = IMP_VFolder::addVFolder($query, $search);
            $GLOBALS['prefs']->setValue('vtrash_id', $vtrash_id);
            $_SESSION['imp']['vtrash_id'] = $vtrash_id;
        } else {
            IMP_VFolder::addVFolder($query, $search, $vtrash_id);
        }

        require_once IMP_BASE . '/lib/IMAP/Tree.php';
        $imptree = &IMP_Tree::singleton();
        $imptree->insertVFolders(array($vtrash_id => _("Virtual Trash")));
    }

    /**
     * Determines whether a virtual folder ID is the Virtual Trash Folder.
     *
     * @param string $id  The virtual folder ID.
     *
     * @return boolean  True if the virutal folder ID is the Virtual Trash
     *                  folder.
     */
    function isVTrashFolder($id)
    {
        $id = IMP_VFolder::_stripIMPLabel($id);
        return ($id == $GLOBALS['prefs']->getValue('vtrash_id'));
    }

    /**
     * Add a virtual INBOX folder for the current user.
     */
    function createVINBOXFolder()
    {
        /* Initialize IMP_Tree. */
        require_once IMP_BASE . '/lib/IMAP/Tree.php';
        $imptree = &IMP_Tree::singleton();

        /* Delete the current Virtual Trash folder, if it exists. */
        $vinbox_id = $GLOBALS['prefs']->getValue('vinbox_id');
        if (!empty($vinbox_id)) {
            IMP_VFolder::deleteVFolder($vinbox_id);
        }

        /* Create Virtual INBOX with nav_poll list. */
        $fl = $imptree->getPollList();
        $flist = array();
        foreach (array_keys($fl) as $mbox) {
            $flist[] = $mbox;
        }

        require_once 'Horde/IMAP/Search.php';

        $query = &new IMAP_Search_query();
        $query->seen(false);

        $search = array('match'         => 'and',
                        'flag'          => array('1' => 2, '2' => 2, '3' => 2, '4' => '0'),
                        'folders'       => $flist,
                        'text'          => '',
                        'text_end'      => '',
                        'mbox'          => '',
                        'date'          => '',
                        'field'         => "",
                        'field_end'     => "",
                        'vfolder_label' => _("Virtual INBOX"));

        if (empty($vinbox_id)) {
            $vinbox_id = IMP_VFolder::addVFolder($query, $search);
            $GLOBALS['prefs']->setValue('vinbox_id', $vinbox_id);
            $_SESSION['imp']['vinbox_id'] = $vinbox_id;
        } else {
            IMP_VFolder::addVFolder($query, $search, $vinbox_id);
        }

        $imptree->insertVFolders(array($vinbox_id => _("Virtual INBOX")));
    }

    /**
     * Determines whether a virtual folder ID is the Virtual INBOX Folder.
     *
     * @param string $id  The virtual folder ID.
     *
     * @return boolean  True if the virutal folder ID is the Virtual INBOX
     *                  folder.
     */
    function isVINBOXFolder($id)
    {
        $id = IMP_VFolder::_stripIMPLabel($id);
        return ($id == $GLOBALS['prefs']->getValue('vinbox_id'));
    }

    /**
     * Is the current active folder an editable Virtual Folder?
     *
     * @return boolean  True if the current folder is both a virtual folder
     *                  and can be edited.
     */
    function isEditableVFolder()
    {
        return (!empty($_SESSION['imp']['in_vfolder']) &&
                !IMP_VFolder::isVTrashFolder($_SESSION['imp']['search']['vfolder_id']) &&
                !IMP_VFolder::isVINBOXFolder($_SESSION['imp']['search']['vfolder_id']));
    }

    /**
     * Remove a virtual folder for the current user.
     *
     * @param string $id  The virtual folder ID to delete.
     */
    function deleteVFolder($id)
    {
        $id = IMP_VFolder::_stripIMPLabel($id);
        if (!empty($_SESSION['imp']['vfolder'][$id])) {
            unset($_SESSION['imp']['vfolder'][$id]);
            IMP_VFolder::_saveVFolderList();

            /* Initialize IMP_Tree. */
            require_once IMP_BASE . '/lib/IMAP/Tree.php';
            $imptree = &IMP_Tree::singleton();
            $imptree->delete(IMP_VFOLDER_PREFIX . $id);
        }
    }

    /**
     * Return a list of IDs and VFolder labels, sorted by the label.
     *
     * @return array  An array with the folder IDs as the key and the labels
     *                as the value.
     */
    function listVFolders()
    {
        $vfolders = array();

        if (!empty($_SESSION['imp']['vfolder'])) {
            foreach ($_SESSION['imp']['vfolder'] as $key => $val) {
                $vfolders[$key] = $val['search']['vfolder_label'];
            }
            natcasesort($vfolders);
        }

        return $vfolders;
    }

    /**
     * Set the necessary search parameters in the session given a virtual
     * folder ID.
     *
     * @param string $id  The virtual folder ID.
     *
     * @return boolean  True on success, false if virtual folder does not
     *                  exist.
     */
    function changeToVFolder($id)
    {
        $id = IMP_VFolder::_stripIMPLabel($id);
        if (!isset($_SESSION['imp']['vfolder'][$id]['ob'])) {
            return false;
        }

        $query = unserialize($_SESSION['imp']['vfolder'][$id]['ob']);
        $search = $_SESSION['imp']['vfolder'][$id]['search'];

        require_once IMP_BASE . '/lib/Search.php';
        IMP_Search::createSearchQuery($query, $search['folders'], $search, $search['vfolder_label'], $id);
        $_SESSION['imp']['in_vfolder'] = $id;

        return true;
    }

    /**
     * Change the label on a vfolder.
     *
     * @param string $id     The virtual folder ID.
     * @param string $label  The new label.
     */
    function relabelVFolder($id, $label)
    {
        $id = IMP_VFolder::_stripIMPLabel($id);
        if (isset($_SESSION['imp']['vfolder'][$id]['search']['vfolder_label'])) {
            $_SESSION['imp']['vfolder'][$id]['search']['vfolder_label'] = $label;
            IMP_VFolder::_saveVFolderList();
        }
    }

    /**
     * Retrieve the UI query object for a given virtual folder ID.
     *
     * @param string $id  The virtual folder ID.
     */
    function retrieveUIQuery($id)
    {
        $id = IMP_VFolder::_stripIMPLabel($id);
        if (isset($_SESSION['imp']['vfolder'][$id]['search'])) {
            return $_SESSION['imp']['vfolder'][$id]['search'];
        }
    }

    /**
     * Returns a link to edit a given virtual folder ID.
     *
     * @param string $id  The virtual folder ID.
     *
     * @return string  The URL to the search page.
     */
    function editVFolderURL($id)
    {
        $id = IMP_VFOLDER_PREFIX . IMP_VFolder::_stripIMPLabel($id);
        return Util::addParameter(Horde::applicationUrl('search.php'), array('edit_vfolder' => 1, 'vfolder' => $id));
    }

    /**
     * Returns a link to delete a given virtual folder ID.
     *
     * @param string $id  The virtual folder ID.
     *
     * @return string  The URL to allow deletion of the virtual folder.
     */
    function deleteVFolderURL($id)
    {
        $id = IMP_VFOLDER_PREFIX . IMP_VFolder::_stripIMPLabel($id);
        return Util::addParameter(Horde::applicationUrl('folders.php'), array('actionID' => 'delete_vfolder', 'vfolder' => $id));
    }

    /**
     * Strip the IMP identifying label from a virtual folder ID.
     *
     * @access private
     *
     * @param string $id  The virtual folder ID.
     *
     * @return string  The virtual folder ID, with any IMP specific identifying
     *                 information stripped off.
     */
    function _stripIMPLabel($id)
    {
        return (strpos($id, IMP_VFOLDER_PREFIX) === 0) ? substr($id, strlen(IMP_VFOLDER_PREFIX)) : $id;
    }

}
